defmodule Cashdesk.Router do
  use Plug.Router

  plug(:match)
  plug(Plug.Parsers, parsers: [:json], json_decoder: Jason)
  plug(:dispatch)

  get "ping" do
    send_resp(conn, 200, "pong")
  end

  post "buy" do
    conn = fetch_query_params(conn)
    {resp_code, response} = handle_buy_request(conn.params)
    send_resp(conn, resp_code, response)
  end

  get "done" do
    {resp_code, response} = handle_done_request()
    send_resp(conn, resp_code, response)
  end

  match _ do
    send_resp(conn, 404, "not found")
  end

  defp handle_buy_request(%{"product" => product, "amount" => amount}) do
    case Cashdesk.buy(product, amount) do
      :ok ->
        {200, "ok"}
      error ->
        {400, Jason.encode!(error)}
    end
  end

  defp handle_buy_request(_) do
    {400, "missing parameters"}
  end

  defp handle_done_request() do
    total = Cashdesk.done()
    {200, Jason.encode!(total)}
  end
end
