defmodule Cashdesk do
  @prices %{"flour" => 100, "egg" => 45, "toilet paper" => 500}

  def start do
    pid = spawn(fn -> init() end)
    Process.register(pid, :cashdesk)
  end

  def init do
    loop(%{total: 0, prices: @prices})
  end

  def buy(product, amount) do
    send(:cashdesk, {:buy, self(), product, amount})
    receive do
      response ->
        response
    after 5000 ->
        :cashdesk_closed
    end
  end

  def done() do
    send(:cashdesk, {:done, self()})
    receive do
      total ->
        total
    end
  end

  defp loop(%{total: total, prices: prices} = state) do
    receive do
      {:buy, customer, product, amount} ->
        case prices[product] do
          nil ->
            send(customer, :not_available)
            loop(state)
          price ->
            pay = amount * price
            send(customer, :ok)
            loop(%{state | total: total + pay})
        end
      {:done, customer} ->
        send(customer, total)
        loop(%{state | total: 0})
    end
  end
end
